@@ -14,7 +14,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2005 Kristian Høgsberg <krh@redhat.com>
-// Copyright (C) 2005, 2007, 2009-2011, 2013, 2017-2021 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2007, 2009-2011, 2013, 2017-2022 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
 // Copyright (C) 2005, 2006, 2008 Brad Hards <bradh@frogmouth.net>
 // Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
@@ -80,7 +80,7 @@ public:
     int numEntries() { return length; };
     // iterator accessor, note it returns a pointer to the internal object, do not free nor delete it
     Object *getValue(int i);
-    GooString *getName(int i);
+    const GooString *getName(int i) const;
 
 private:
     struct Entry
@@ -111,7 +111,7 @@ class POPPLER_PRIVATE_EXPORT Catalog
 {
 public:
     // Constructor.
-    Catalog(PDFDoc *docA);
+    explicit Catalog(PDFDoc *docA);
 
     // Destructor.
     ~Catalog();
@@ -136,7 +136,7 @@ public:
 
     // Return the contents of the metadata stream, or NULL if there is
     // no metadata.
-    GooString *readMetadata();
+    std::unique_ptr<GooString> readMetadata();
 
     // Return the structure tree root object.
     StructTreeRoot *getStructTreeRoot();
@@ -174,7 +174,7 @@ public:
     int numDestNameTree() { return getDestNameTree()->numEntries(); }
 
     // Get the i'th named destination name in name-tree
-    GooString *getDestNameTreeName(int i) { return getDestNameTree()->getName(i); }
+    const GooString *getDestNameTreeName(int i) { return getDestNameTree()->getName(i); }
 
     // Get the i'th named destination link destination in name-tree
     std::unique_ptr<LinkDest> getDestNameTreeDest(int i);
@@ -183,7 +183,7 @@ public:
     int numEmbeddedFiles() { return getEmbeddedFileNameTree()->numEntries(); }
 
     // Get the i'th file embedded (at the Document level) in the document
-    FileSpec *embeddedFile(int i);
+    std::unique_ptr<FileSpec> embeddedFile(int i);
 
     // Is there an embedded file with the given name?
     bool hasEmbeddedFile(const std::string &fileName);
@@ -195,7 +195,7 @@ public:
 
     // Get the number of javascript scripts
     int numJS() { return getJSNameTree()->numEntries(); }
-    GooString *getJSName(int i) { return getJSNameTree()->getName(i); }
+    const GooString *getJSName(int i) { return getJSNameTree()->getName(i); }
 
     // Get the i'th JavaScript script (at the Document level) in the document
     GooString *getJS(int i);
